#include "ogreapplication.h"

#include <CEGUI/CEGUISchemeManager.h>

using namespace Ogre;
using namespace std;

namespace
{
  void
  parseResourceLocationFrom(const String& resourcesFileName)
  {
    // Load resource paths from config file
    ConfigFile cf;
    cf.load(resourcesFileName);    
    // Go through all sections & settings in the file
    ConfigFile::SectionIterator seci = cf.getSectionIterator();    
    while (seci.hasMoreElements())
    {
      String secName = seci.peekNextKey();
      ConfigFile::SettingsMultiMap* settings = seci.getNext();
      for (ConfigFile::SettingsMultiMap::iterator i = settings->begin();
					 i != settings->end(); ++i)
      {
				String typeName = i->first;
				String archName = i->second;
				ResourceGroupManager::getSingleton().addResourceLocation(
					archName, typeName, secName);
      }
    }
  }
}
//************************************************************
// Implementation of OgreApplication
//************************************************************
OgreApplication::OgreApplication(Ogre::String name)
  : mName(name)
	, mRoot(NULL)
	, mWindow(NULL)
	, mEventProcessor(NULL)
	, mGUIRenderer(NULL)
	, mGUISystem(NULL)
	, mSelected(NULL)
	, mQuitRequested(false)
	, mAnimationsPaused(false)
{
}
OgreApplication::~OgreApplication()
{
  if (mEventProcessor) delete mEventProcessor;
  if (mTrackball)      delete mTrackball;
  if (mGUISystem)      delete mGUISystem;
  if (mGUIRenderer)    delete mGUIRenderer;
  if (mRoot)           delete mRoot;
}
void
OgreApplication::addResourceLocationFrom(const String& fileName)
{
  mResourcesFileNames.push_back(fileName);  
}
void
OgreApplication::create(const String& pluginsFileName,
												const String& configFileName,
												const String& logFileName,
												SceneType sceneType)
{
  //////////////////////////////////////////////////////////////
  // Initialization of Root and RenderWindow
  //////////////////////////////////////////////////////////////
  mRoot = new Root(pluginsFileName,configFileName,logFileName);
  if (!mRoot->restoreConfig())
  {
    if (!mRoot->showConfigDialog())
    {      
      throw Exception(0,"user quitted config dialog","OgreApplication::init()",__FILE__,__LINE__);
    }
  }
  mWindow = mRoot->initialise(true,mName);
  //////////////////////////////////////////////////////////////
  // Initializing resources
  //////////////////////////////////////////////////////////////
  for_each(mResourcesFileNames.begin(),mResourcesFileNames.end(),parseResourceLocationFrom);
  ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
  //////////////////////////////////////////////////////////////
  // Initialization of Scene
  //////////////////////////////////////////////////////////////
  mSceneManager = mRoot->createSceneManager(sceneType,"sceneManager");
  mCamera = mSceneManager->createCamera("Camera");
  mCamera->setPosition(Vector3(0.0f,0.0f,500.0f));
  mCamera->lookAt(Vector3(0.0f,0.0f,0.0f));
  mCamera->setNearClipDistance(5.0f);
  mCamera->setFarClipDistance(5000.0f);
  ////////////////////////////////////////////////////////////
  // Binding of Scene and RenderWindow
  ////////////////////////////////////////////////////////////
  mViewport = mWindow->addViewport(mCamera);
  mViewport->setBackgroundColour(ColourValue(0.0f,0.0f,0.0f));
  mCamera->setAspectRatio(Real(mViewport->getActualWidth()) /
													Real(mViewport->getActualHeight()));
  ////////////////////////////////////////////////////////////
  // Initialize GUI
  ////////////////////////////////////////////////////////////
  mGUIRenderer = new CEGUI::OgreCEGUIRenderer(mWindow,Ogre::RENDER_QUEUE_OVERLAY,
																							false,3000,
																							mSceneManager);
  mGUISystem = new CEGUI::System(mGUIRenderer);
  
  CEGUI::SchemeManager::getSingleton().loadScheme((CEGUI::utf8*)"TaharezLookSkin.scheme");
  mGUISystem->setDefaultMouseCursor(reinterpret_cast<const CEGUI::utf8*>("TaharezLook"),
																		reinterpret_cast<const CEGUI::utf8*>("MouseArrow"));
  CEGUI::MouseCursor::getSingleton().setImage(CEGUI::System::getSingleton().getDefaultMouseCursor());
  ////////////////////////////////////////////////////////////
  // Managing input
  ////////////////////////////////////////////////////////////
  mRoot->addFrameListener(this);

  mTrackball = new Trackball(mGUIRenderer,mGUISystem,mCamera,mSceneManager);
    
  mEventProcessor = new EventProcessor();
  mEventProcessor->addKeyListener(this);
  mEventProcessor->addMouseListener(mTrackball);
  mEventProcessor->addMouseMotionListener(mTrackball);
  //////////////////////////////////////////////////////////////
  //  Debug Overlay
  //////////////////////////////////////////////////////////////
  mDebugPanelOverlay = OverlayManager::getSingleton().getByName("Core/DebugOverlay");
  //////////////////////////////////////////////////////////////
  // Ray/Scene query
  //////////////////////////////////////////////////////////////
  mRaySceneQuery = mSceneManager->createRayQuery(Ray());
  mEventProcessor->addMouseListener(this);
  //////////////////////////////////////////////////////////////
  //  User initialization (subclassed)
  //////////////////////////////////////////////////////////////  
  init();
}
void
OgreApplication::manage(AnimationState* s,bool enabled,bool loop)
{
  mAnimationStates.push_back(s);
  s->setLoop(loop);
  s->setEnabled(enabled);  
}
void
OgreApplication::run()
{
  mEventProcessor->initialise(mWindow);
  mEventProcessor->startProcessingEvents();
  mRoot->startRendering();
}
void
OgreApplication::keyClicked(KeyEvent *e)
{
  if (e->getKey() == KC_F1)
  {
    if (mDebugPanelOverlay) 
    {
      if (mDebugPanelOverlay->isVisible())
				mDebugPanelOverlay->hide();
      else
				mDebugPanelOverlay->show();      
    }
    e->consume();
  }
  else if (e->getKey() == KC_ESCAPE)
  {
    requestQuit();
    e->consume();
  }
  else if (e->getKey() == KC_SYSRQ)
  {
    static size_t nb = 0;
    String fileName = "screenshot_"+StringConverter::toString(nb++)+".png";
    mWindow->writeContentsToFile(fileName);
    LogManager::getSingleton().logMessage("screenshot saved in "+fileName);
    e->consume();
  }
  else if (e->getKey() == KC_SPACE)
  {
    setAnimationPaused(!mAnimationsPaused);
    e->consume();
  }
}
void
OgreApplication::keyPressed(KeyEvent *e)
{
}
void
OgreApplication::keyReleased(KeyEvent *e)
{
}
bool
OgreApplication::frameStarted(const FrameEvent& evt)
{
  if (!mAnimationsPaused)
  {
    for (deque<AnimationState*>::const_iterator iter=mAnimationStates.begin();iter != mAnimationStates.end();++iter)
    {
      AnimationState* a = *iter;
      if (a->getEnabled()) a->addTime(evt.timeSinceLastFrame);
    }
  }
  return true;
}
bool
OgreApplication::frameEnded(const FrameEvent& evt)
{
  static const String currFps  = "Current FPS: ";
  static const String avgFps   = "Average FPS: ";
  static const String bestFps  = "Best FPS: ";
  static const String worstFps = "Worst FPS: ";
  static const String tris     = "Triangle Count: ";

  if (mDebugPanelOverlay && mDebugPanelOverlay->isVisible())
  {
    OverlayElement* guiCurr  = OverlayManager::getSingleton().getOverlayElement("Core/CurrFps");
    OverlayElement* guiAvg   = OverlayManager::getSingleton().getOverlayElement("Core/AverageFps");
    OverlayElement* guiBest  = OverlayManager::getSingleton().getOverlayElement("Core/BestFps");
    OverlayElement* guiWorst = OverlayManager::getSingleton().getOverlayElement("Core/WorstFps");
    
    const RenderTarget::FrameStats& stats = mWindow->getStatistics();
    
    guiCurr->setCaption(currFps + StringConverter::toString(stats.lastFPS));
    guiAvg->setCaption(avgFps + StringConverter::toString(stats.avgFPS));
    guiBest->setCaption(bestFps + StringConverter::toString(stats.bestFPS)
												+" "+StringConverter::toString(stats.bestFrameTime)+" ms");
    guiWorst->setCaption(worstFps + StringConverter::toString(stats.worstFPS)
												 +" "+StringConverter::toString(stats.worstFrameTime)+" ms");
    
    OverlayElement* guiTris = OverlayManager::getSingleton().getOverlayElement("Core/NumTris");
    guiTris->setCaption(tris + StringConverter::toString(stats.triangleCount));

    OverlayElement* guiDbg = OverlayManager::getSingleton().getOverlayElement("Core/DebugText");
    guiDbg->setCaption(mWindow->getDebugText());    
  }
  
  return !mQuitRequested;
}
void
OgreApplication::init()
{
}
void
OgreApplication::requestQuit()
{
  mQuitRequested = true;
}
bool
OgreApplication::getAnimationPaused() const
{
  return mAnimationsPaused;
}
void
OgreApplication::setAnimationPaused(bool b)
{
  mAnimationsPaused = b;
}
void
OgreApplication::mousePressed(MouseEvent*)
{
}
void
OgreApplication::mouseReleased(MouseEvent* e)
{
  if (e->getButtonID() == InputEvent::BUTTON0_MASK && e->isShiftDown()) // Shift+Left 
  {
    // Deselect old one if any
    if (mSelected) mSelected->getParentSceneNode()->showBoundingBox(false);
    mWindow->setDebugText("");
    // Use a ray scene query to find where we clicked
    Ray mouseRay = mCamera->getCameraToViewportRay(e->getX(),e->getY());
    mRaySceneQuery->setRay(mouseRay);
    mRaySceneQuery->setSortByDistance(true);
    RaySceneQueryResult &result = mRaySceneQuery->execute();
    if (!result.empty())
    {
      RaySceneQueryResultEntry& re = result.front();
      if (re.movable && re.movable->getMovableType() == "Entity")
      {
				mSelected = (Entity*)(re.movable);
				mSelected->getParentSceneNode()->showBoundingBox(true);
				mWindow->setDebugText("Selected "+mSelected->getName());
      }
    }
  }
}
void
OgreApplication::mouseClicked(MouseEvent*)
{
}
void
OgreApplication::mouseEntered(MouseEvent*)
{
}
void
OgreApplication::mouseExited(MouseEvent*)
{
}

